using System;
using System.Text;
#if PocketPC
using Microsoft.WindowsCE.Forms;
#endif
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace c_tt
{

    public enum ClientEvent
    {
        WM_TEAMTALK_DUMMY_FIRST = 0x0400 + 449,
        WM_TEAMTALK_CON_SUCCESS,
        WM_TEAMTALK_CON_FAILED,
        WM_TEAMTALK_CON_LOST,
        WM_TEAMTALK_CON_P2P,
        WM_TEAMTALK_CMD_PROCESSING,
        WM_TEAMTALK_CMD_MYSELF_LOGGEDIN,
        WM_TEAMTALK_CMD_MYSELF_LOGGEDOUT,
        WM_TEAMTALK_CMD_MYSELF_JOINED,
        WM_TEAMTALK_CMD_MYSELF_LEFT,
        WM_TEAMTALK_CMD_MYSELF_KICKED,
        WM_TEAMTALK_CMD_USER_LOGGEDIN,
        WM_TEAMTALK_CMD_USER_LOGGEDOUT,
        WM_TEAMTALK_CMD_USER_UPDATE,
        WM_TEAMTALK_CMD_USER_JOINED,
        WM_TEAMTALK_CMD_USER_LEFT,
        WM_TEAMTALK_CMD_USER_TEXTMSG,
        WM_TEAMTALK_CMD_CHANNEL_NEW,
        WM_TEAMTALK_CMD_CHANNEL_UPDATE,
        WM_TEAMTALK_CMD_CHANNEL_REMOVE,
        WM_TEAMTALK_CMD_SERVER_UPDATE,
        WM_TEAMTALK_CMD_FILE_NEW,
        WM_TEAMTALK_CMD_FILE_REMOVE,
        WM_TEAMTALK_CMD_ERROR,
        WM_TEAMTALK_CMD_SUCCESS,
        WM_TEAMTALK_USER_TALKING,
        WM_TEAMTALK_USER_VIDEOFRAME,
        WM_TEAMTALK_USER_AUDIOFILE,
        WM_TEAMTALK_INTERNAL_ERROR,
        WM_TEAMTALK_VOICE_ACTIVATION,
        WM_TEAMTALK_STREAM_AUDIOFILE_USER,
        WM_TEAMTALK_STREAM_AUDIOFILE_CHANNEL,
        WM_TEAMTALK_HOTKEY,
        WM_TEAMTALK_HOTKEY_TEST,
        WM_TEAMTALK_FILETRANSFER,
        WM_TEAMTALK_USER_AUDIOBLOCK,
        WM_TEAMTALK_USER_DESKTOPWINDOW,
        WM_TEAMTALK_DESKTOPWINDOW_TRANSFER,
        WM_TEAMTALK_USER_DESKTOPCURSOR,
        WM_TEAMTALK_CON_MAX_PAYLOAD_UPDATED,
        WM_TEAMTALK_STREAM_MEDIAFILE_CHANNEL,
        WM_TEAMTALK_DUMMY_LAST
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
    public struct TTMessage
    {
        public ClientEvent wmMsg;
        public uint wParam;
        public uint lParam;
    }

    enum TTStructType : uint
    {
        __AUDIOCODEC,
        __BANNEDUSER,
        __CAPTUREFORMAT,
        __CELTCODEC,
        __CHANNEL,
        __CLIENTSTATISTICS,
        __FILEINFO,
        __FILETRANSFER,
        __SERVERPROPERTIES,
        __SERVERSTATISTICS,
        __SOUNDDEVICE,
        __SPEEXCODEC,
        __TEXTMESSAGE,
        __THEORACODEC,
        __TTMESSAGE,
        __USER,
        __USERACCOUNT,
        __USERSTATISTICS,
        __VIDEOCAPTUREDEVICE,
        __VIDEOCODEC,
        __AUDIOCONFIG,
        __CELTVBRCODEC,
        __SPEEXVBRCODEC,
        __VIDEOFRAME,
        __AUDIOBLOCK,
        __AUDIOFORMAT,
        __MEDIAFILEINFO
    }

#if PocketPC
    class MyEventHandler : MessageWindow
#else
    class MyEventHandler : System.Windows.Forms.UserControl
#endif
    {
        public MyEventHandler(BearWare.TeamTalk4 tt)
        {
            m_tt = tt;
        }

        protected BearWare.TeamTalk4 m_tt;
        protected override void WndProc(ref Message m)
        {
            if (m.Msg >= (int)ClientEvent.WM_TEAMTALK_DUMMY_FIRST &&
                m.Msg <= (int)ClientEvent.WM_TEAMTALK_DUMMY_LAST)
            {
                TTMessage msg;
                msg.wmMsg = (ClientEvent)m.Msg;
                msg.wParam = (uint)m.WParam.ToInt32();
                msg.lParam = (uint)m.LParam.ToInt32();
                m_tt.processMsg(ref msg);
            }
            else
                base.WndProc(ref m);
        }
    }

    class TT4
    {
#if ENABLE_ENCRYPTION
        public const string dllname = "TeamTalk4Pro.dll";
        public const string mgtdllname = "TeamTalk4Pro.NET.dll";
#else
        public const string dllname = "TeamTalk4.dll";
        public const string mgtdllname = "TeamTalk4.NET.dll";
#endif

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern IntPtr TT_GetVersion();
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern IntPtr TT_InitTeamTalk(IntPtr hWnd);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern IntPtr TT_InitTeamTalkPoll();
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_CloseTeamTalk(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetMessage(IntPtr lpTTInstance,
                                               out TTMessage pMsg,
                                               ref int pnWaitMs);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern BearWare.ClientFlag TT_GetFlags(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetLicenseInformation([MarshalAs(UnmanagedType.LPWStr)] string szRegName,
                                                           long nRegKey);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetDefaultSoundDevices(IntPtr lpTTInstance,
                                                            out int lpnInputDeviceID,
                                                            out int lpnOutputDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetDefaultSoundDevicesEx(BearWare.SoundSystem nSoundSystem,
                                                              out int lpnInputDeviceID,
                                                              out int lpnOutputDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetSoundInputDevices(IntPtr lpTTInstance,
                                                 [In, Out] BearWare.SoundDevice[] lpSoundDevices,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetSoundInputDevices")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetSoundInputDevices_NULL(IntPtr lpTTInstance,
                                                 IntPtr lpSoundDevices,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetSoundOutputDevices(IntPtr lpTTInstance,
                                                      [In, Out] BearWare.SoundDevice [] lpSoundDevices,
                                                     ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetSoundOutputDevices")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetSoundOutputDevices_NULL(IntPtr lpTTInstance,
                                                      IntPtr lpSoundDevices,
                                                     ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_InitSoundInputDevice(IntPtr lpTTInstance,
                                                     int nInputDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_InitSoundOutputDevice(IntPtr lpTTInstance,
                                                      int nOutputDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_InitSoundDuplexDevices(IntPtr lpTTInstance,
                                                      int nInputDeviceID,
                                                      int nOutputDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_CloseSoundInputDevice(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_CloseSoundOutputDevice(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_CloseSoundDuplexDevices(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_RestartSoundSystem();
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StartSoundLoopbackTest(IntPtr lpTTInstance,
                                                       int nInputDeviceID,
                                                       int nOutputDeviceID,
                                                       int nSampleRate,
                                                       int nChannels);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StartSoundLoopbackTestEx(IntPtr lpTTInstance,
                                        int nInputDeviceID, int nOutputDeviceID,
                                        int nSampleRate, int nChannels,
                                        ref BearWare.AudioConfig lpAudioConfig, bool bEchoCancel);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StopSoundLoopbackTest(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetSoundInputLevel(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetSoundInputGainLevel(IntPtr lpTTInstance,
                                                       int nLevel);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetSoundInputGainLevel(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetSoundOutputVolume(IntPtr lpTTInstance,
                                                     int nVolume);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetSoundOutputVolume(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetSoundOutputMute(IntPtr lpTTInstance,
                                                   bool bMuteAll);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_EnableDenoising(IntPtr lpTTInstance,
                                                bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetDenoising(IntPtr lpTTInstance,
                                                bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetDenoiseLevel(IntPtr lpTTInstance,
                                                int nLevel);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetDenoiseLevel(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_EnableEchoCancellation(IntPtr lpTTInstance,
                                          bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_EnableAGC(IntPtr lpTTInstance,
                                          bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetAGCSettings(IntPtr lpTTInstance,
                                               int nGainLevel,
                                               int nMaxIncrement,
                                               int nMaxDecrement,
                                               int nMaxGain);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetAGCSettings(IntPtr lpTTInstance,
                                               out int lpnGainLevel,
                                               out int lpnMaxIncrement,
                                               out int lpnMaxDecrement,
                                               out int lpnMaxGain);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Enable3DSoundPositioning(IntPtr lpTTInstance,
                                                         bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_AutoPositionUsers(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_EnableAudioBlockEvent(IntPtr lpTTInstance, bool bEnable);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_EnableVoiceActivation(IntPtr lpTTInstance,
                                                      bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetVoiceActivationLevel(IntPtr lpTTInstance,
                                                        int nLevel);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetVoiceActivationLevel(IntPtr lpTTInstance);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetVoiceActivationStopDelay(IntPtr lpTTInstance,
                                                                 int nDelayMSec);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetVoiceActivationStopDelay(IntPtr lpTTInstance);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StartRecordingMuxedAudioFile(IntPtr lpTTInstance,
                                                       ref BearWare.AudioCodec lpAudioCodec,
                    [MarshalAs(UnmanagedType.LPWStr)] string szAudioFileName,
                                                       BearWare.AudioFileFormat aff);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StopRecordingMuxedAudioFile(IntPtr lpTTInstance);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetVideoCaptureDevices")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetVideoCaptureDevices_NULL(IntPtr lpTTInstance,
                                                       IntPtr lpVideoDevices,
                                                       ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetVideoCaptureDevices(IntPtr lpTTInstance,
                                                       [In, Out] BearWare.VideoCaptureDevice[] lpVideoDevices,
                                                       ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_InitVideoCaptureDevice(IntPtr lpTTInstance,
                                                       [MarshalAs(UnmanagedType.LPWStr)] string szDeviceID,
                                                      ref BearWare.CaptureFormat lpCaptureFormat,
                                                      ref BearWare.VideoCodec lpVideoCodec);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_CloseVideoCaptureDevice(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_PaintVideoFrame(IntPtr lpTTInstance,
                                                int nUserID, IntPtr hDC, int nPosX,
                                                int nPosY, int nWidth, int nHeight);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_PaintVideoFrameEx(IntPtr lpTTInstance,
                                                       int nUserID, IntPtr hDC, int XDest,
                                                       int YDest, int nDestWidth, int nDestHeight,
                                                       int XSrc, int YSrc, int nSrcWidth, int nSrcHeight);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUserVideoFrame(IntPtr lpTTInstance,
                                                  int nUserID,
                                                  IntPtr lpPicBuffer,
                                                  int nPicBufSize,
                                                  out BearWare.CaptureFormat lpCaptureFormat);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_AcquireUserVideoFrame(IntPtr lpTTInstance,
                                              int nUserID,
                                              ref BearWare.VideoFrame lpVideoFrame);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_ReleaseUserVideoFrame(IntPtr lpTTInstance,
                                                      int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_EnableTransmission(IntPtr lpTTInstance,
                                                   BearWare.TransmitType uTxType,
                                                   bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_IsTransmitting(IntPtr lpTTInstance,
                                               BearWare.TransmitType uTxType);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StartStreamingAudioFileToUser(IntPtr lpTTInstance,
                                                              int nUserID,
                                                              [MarshalAs(UnmanagedType.LPWStr)] string szAudioFilePath);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StopStreamingAudioFileToUser(IntPtr lpTTInstance,
                                                             int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StartStreamingAudioFileToChannel(IntPtr lpTTInstance,
                                                                 int nChannelID,
                                                                 [MarshalAs(UnmanagedType.LPWStr)] string szAudioFilePath);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StopStreamingAudioFileToChannel(IntPtr lpTTInstance,
                                                                int nChannelID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StartStreamingMediaFileToChannel(IntPtr lpTTInstance,
                                                                     [MarshalAs(UnmanagedType.LPWStr)] string szMediaFilePath,
                                                                     ref BearWare.VideoCodec lpVideoCodec,
                                                                     BearWare.TransmitType uTxType);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_StopStreamingMediaFileToChannel(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetMediaFileInfo([MarshalAs(UnmanagedType.LPWStr)] string szMediaFilePath,
                                                      out BearWare.MediaFileInfo pMediaFileInfo);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_SendDesktopWindow(IntPtr lpTTInstance,
                                               IntPtr lpBitmap, 
                                               int nBitmapSize,
                                               ref BearWare.DesktopWindow lpDesktopWindow,
                                               BearWare.BitmapFormat nConvertBmpFormat);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_CloseDesktopWindow(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern IntPtr TT_Palette_GetColorTable(BearWare.BitmapFormat nBmpPalette,
                                                             int nIndex);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern IntPtr TT_Windows_GetDesktopActiveHWND();
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern IntPtr TT_Windows_GetDesktopHWND();
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Windows_GetDesktopWindowHWND(int nIndex,
                                                                  ref IntPtr lpHWnd);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Windows_GetWindow(IntPtr hWnd,
                                                       ref BearWare.ShareWindow lpShareWindow);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_SendDesktopWindowFromHWND(IntPtr lpTTInstance,
                                                       IntPtr hWnd,
                                                       BearWare.BitmapFormat nBitmapFormat,
                                                       BearWare.DesktopProtocol nDesktopProtocol);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_PaintDesktopWindow(IntPtr lpTTInstance,
                                                        int nUserID,
                                                        IntPtr hDC,
                                                        int XDest,
                                                        int YDest,
                                                        int nDestWidth,
                                                        int nDestHeight);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_PaintDesktopWindowEx(IntPtr lpTTInstance,
                                                          int nUserID,
                                                         IntPtr hDC,
                                                         int XDest,
                                                         int YDest,
                                                         int nDestWidth,
                                                         int nDestHeight,
                                                         int XSrc,
                                                         int YSrc,
                                                         int nSrcWidth,
                                                         int nSrcHeight);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SendDesktopCursorPosition(IntPtr lpTTInstance,
                                                                int nUserID,
                                                                  int nPosX,
                                                                  int nPosY);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUserDesktopWindow(IntPtr lpTTInstance, 
                                                             int nUserID, 
                                                             IntPtr lpBitmap, 
                                                             ref int lpnBitmapSize,
                                                             ref BearWare.DesktopWindow lpDesktopWindow);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
            public static extern bool TT_GetUserDesktopCursor(IntPtr lpTTInstance, 
                                                 int nSrcUserID,
                                                 int nDestUserID,
                                                 ref int lpnPosX,
                                                 ref int lpnPosY);    

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Connect(IntPtr lpTTInstance,
                                        [MarshalAs(UnmanagedType.LPWStr)] string szHostAddress,
                                        int nTcpPort,
                                        int nUdpPort,
                                        int nLocalTcpPort,
                                        int nLocalUdpPort);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_ConnectEx(IntPtr lpTTInstance,
                                          [MarshalAs(UnmanagedType.LPWStr)] string szHostAddress,
                                          int nTcpPort,
                                          int nUdpPort,
                                          [MarshalAs(UnmanagedType.LPWStr)] string szBindIPAddr,
                                          int nLocalTcpPort,
                                          int nLocalUdpPort);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_ConnectNonEncrypted(IntPtr lpTTInstance,
                                          [MarshalAs(UnmanagedType.LPWStr)] string szHostAddress,
                                          int nTcpPort,
                                          int nUdpPort,
                                          [MarshalAs(UnmanagedType.LPWStr)] string szBindIPAddr,
                                          int nLocalTcpPort,
                                          int nLocalUdpPort);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Disconnect(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_QueryMaxPayload(IntPtr lpTTInstance, int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetKeepAliveInterval(IntPtr lpTTInstance,
                                                     int nTcpPingIntervalSec,
                                                     int nUdpPingIntervalSec);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetKeepAliveInterval(IntPtr lpTTInstance,
                                                        out int lpnTcpPingIntervalSec,
                                                        out int lpnUdpPingIntervalSec);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetServerTimeout(IntPtr lpTTInstance,
                                                 int nTimeoutSec);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetServerTimeout(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_EnablePeerToPeer(IntPtr lpTTInstance,
                                                 BearWare.TransmitType mask,
                                                 bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetStatistics(IntPtr lpTTInstance,
                                              out BearWare.ClientStatistics lpStats);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetPacketSize(ref BearWare.AudioCodec lpCodec);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoLogin(IntPtr lpTTInstance,
                                         [MarshalAs(UnmanagedType.LPWStr)] string szNickname,
                                         [MarshalAs(UnmanagedType.LPWStr)] string szServerPassword,
                                         [MarshalAs(UnmanagedType.LPWStr)] string szUsername,
                                         [MarshalAs(UnmanagedType.LPWStr)] string szPassword);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoLogout(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoJoinChannel(IntPtr lpTTInstance,
                                               ref BearWare.Channel lpChannel);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoJoinChannelByID(IntPtr lpTTInstance,
                                                   int nChannelID,
                                                   [MarshalAs(UnmanagedType.LPWStr)] string szPassword);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoLeaveChannel(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoChangeNickname(IntPtr lpTTInstance,
                                              [MarshalAs(UnmanagedType.LPWStr)] string szNewNick);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoChangeStatus(IntPtr lpTTInstance,
                                                int nStatusMode,
                                                [MarshalAs(UnmanagedType.LPWStr)] string szStatusMessage);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoTextMessage(IntPtr lpTTInstance,
                                               ref BearWare.TextMessage lpTextMessage);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoChannelOp(IntPtr lpTTInstance,
                                             int nUserID,
                                             int nChannelID,
                                             bool bMakeOperator);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoChannelOpEx(IntPtr lpTTInstance,
                                               int nUserID,
                                               int nChannelID,
                                               [MarshalAs(UnmanagedType.LPWStr)] string szOpPassword,
                                               bool bMakeOperator);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoKickUser(IntPtr lpTTInstance,
                                            int nUserID,
                                            int nChannelID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoSendFile(IntPtr lpTTInstance,
                                            int nChannelID,
                                            [MarshalAs(UnmanagedType.LPWStr)] string szLocalFilePath);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoRecvFile(IntPtr lpTTInstance,
                                            int nChannelID,
                                            int nFileID,
                                            [MarshalAs(UnmanagedType.LPWStr)] string szLocalFilePath);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoDeleteFile(IntPtr lpTTInstance,
                                              int nChannelID,
                                              int nFileID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoSubscribe(IntPtr lpTTInstance,
                                             int nUserID,
                                             BearWare.Subscription uSubscriptions);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoUnsubscribe(IntPtr lpTTInstance,
                                               int nUserID,
                                               BearWare.Subscription uSubscriptions);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoMakeChannel(IntPtr lpTTInstance,
                                               ref BearWare.Channel lpChanInfo);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoUpdateChannel(IntPtr lpTTInstance,
                                                 ref BearWare.Channel lpChanInfo);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoRemoveChannel(IntPtr lpTTInstance,
                                                 int nChannelID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoMoveUser(IntPtr lpTTInstance,
                                            int nUserID,
                                            int nChannelID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoUpdateServer(IntPtr lpTTInstance,
                                                ref BearWare.ServerProperties lpServerInfo);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoListUserAccounts(IntPtr lpTTInstance,
                                                    int nIndex,
                                                    int nCount);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoNewUserAccount(IntPtr lpTTInstance,
                                                  ref BearWare.UserAccount lpUserAccount);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoDeleteUserAccount(IntPtr lpTTInstance,
                                                     [MarshalAs(UnmanagedType.LPWStr)] string szUsername);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoBanUser(IntPtr lpTTInstance,
                                           int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoBanIPAddress(IntPtr lpTTInstance,
                                                   [MarshalAs(UnmanagedType.LPWStr)] string szIpAddress);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoUnBanUser(IntPtr lpTTInstance,
                                             [MarshalAs(UnmanagedType.LPWStr)] string szIpAddress);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoListBans(IntPtr lpTTInstance,
                                            int nIndex,
                                            int nCount);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoSaveConfig(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoQueryServerStats(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DoQuit(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetServerProperties(IntPtr lpTTInstance,
                                                    out BearWare.ServerProperties lpProperties);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetServerStatistics(IntPtr lpTTInstance,
                                                    out BearWare.ServerStatistics lpStatistics);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetServerUsers")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetServerUsers_NULL(IntPtr lpTTInstance,
                                                IntPtr lpUserIDs,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetServerUsers(IntPtr lpTTInstance,
                                                [In, Out] int[] lpUserIDs,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetRootChannelID(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetMyChannelID(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetChannel(IntPtr lpTTInstance,
                                           int nChannelID,
                                           out BearWare.Channel lpChannel);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetChannelPath(IntPtr lpTTInstance,
                                               int nChannelID,
                                               IntPtr szChannelPath);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetChannelIDFromPath(IntPtr lpTTInstance,
                                                      [MarshalAs(UnmanagedType.LPWStr)] string szChannelPath);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetChannelUsers")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetChannelUsers_NULL(IntPtr lpTTInstance,
                                                int nChannelID,
                                                IntPtr lpUserIDs,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetChannelUsers(IntPtr lpTTInstance,
                                                int nChannelID,
                                                [In, Out] int[] lpUserIDs,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetChannelFiles")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetChannelFiles_NULL(IntPtr lpTTInstance,
                                                int nChannelID,
                                                IntPtr lpFileInfos,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetChannelFiles(IntPtr lpTTInstance,
                                                int nChannelID,
                                                [In, Out] BearWare.FileInfo[] lpFileInfos,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetChannelFileInfo(IntPtr lpTTInstance,
                                                   int nChannelID,
                                                   int nFileID,
                                                   out BearWare.FileInfo lpFileInfo);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_IsChannelOperator(IntPtr lpTTInstance,
                                                  int nUserID,
                                                  int nChannelID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetServerChannels")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetServerChannels_NULL(IntPtr lpTTInstance,
                                                IntPtr lpUserIDs,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetServerChannels(IntPtr lpTTInstance,
                                                [In, Out] int[] lpUserIDs,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetMyUserID(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetMyUserAccount(IntPtr lpTTInstance,
                                                 out BearWare.UserAccount lpUserAccount);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern BearWare.UserType TT_GetMyUserType(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetMyUserData(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUser(IntPtr lpTTInstance,
                                        int nUserID, out BearWare.User lpUser);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUserStatistics(IntPtr lpTTInstance,
                                                  int nUserID,
                                                  out BearWare.UserStatistics lpStats);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUserByUsername(IntPtr lpTTInstance,
                                                  [MarshalAs(UnmanagedType.LPWStr)] string szUsername,
                                                 out BearWare.User lpUser);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetTextMessage(IntPtr lpTTInstance,
                                               int nMsgID,
                                               bool bRemoveMsg,
                                               out BearWare.TextMessage lpTextMessage);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserVolume(IntPtr lpTTInstance,
                                              int nUserID, int nVolume);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetUserVolume(IntPtr lpTTInstance,
                                               int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserGainLevel(IntPtr lpTTInstance,
                                                 int nUserID,
                                                 int nGainLevel);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetUserGainLevel(IntPtr lpTTInstance,
                                                  int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserMute(IntPtr lpTTInstance,
                                            int nUserID, bool bMute);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserStoppedTalkingDelay(IntPtr lpTTInstance,
                                                                int nUserID,
                                                                int nDelayMSec);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_GetUserStoppedTalkingDelay(IntPtr lpTTInstance,
                                                                int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserPosition(IntPtr lpTTInstance,
                                                int nUserID,
                                                float x,
                                                float y,
                                                float z);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUserPosition(IntPtr lpTTInstance,
                                                int nUserID,
                                                out float x,
                                                out float y,
                                                out float z);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserStereo(IntPtr lpTTInstance,
                                              int nUserID,
                                              bool bLeftSpeaker,
                                              bool bRightSpeaker);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUserStereo(IntPtr lpTTInstance,
                                              int nUserID,
                                              out bool lpbLeftSpeaker,
                                              out bool lpbRightSpeaker);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserAudioFolder(IntPtr lpTTInstance,
                                                   int nUserID,
                                                   [MarshalAs(UnmanagedType.LPWStr)] string szFolderPath,
                                                   [MarshalAs(UnmanagedType.LPWStr)] string szFileNameVars,
                                                   BearWare.AudioFileFormat uAFF);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_SetUserMediaBufferSize(IntPtr lpTTInstance,
                                                            int nUserID,
                                                            int nMSec);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_AcquireUserAudioBlock(IntPtr lpTTInstance,
                                                           int nUserID,
                                                           ref BearWare.AudioBlock lpAudioBlock);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_ReleaseUserAudioBlock(IntPtr lpTTInstance, int nUserID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_ReleaseAllAudioBlocks(IntPtr lpTTInstance);

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetFileTransferInfo(IntPtr lpTTInstance,
                                                    int nTransferID,
                                                    out BearWare.FileTransfer lpTransfer);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_CancelFileTransfer(IntPtr lpTTInstance,
                                                   int nTransferID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetBannedUsers")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetBannedUsers_NULL(IntPtr lpTTInstance,
                                               IntPtr lpBannedUsers,
                                               ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetBannedUsers(IntPtr lpTTInstance,
                                               [In, Out] BearWare.BannedUser[] lpBannedUsers,
                                               ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode, EntryPoint = "TT_GetUserAccounts")] /* Windows CE NULL pointer workaround */
        public static extern bool TT_GetUserAccounts_NULL(IntPtr lpTTInstance,
                                                IntPtr lpUserAccounts,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_GetUserAccounts(IntPtr lpTTInstance,
                                                [In, Out] BearWare.UserAccount[] lpUserAccounts,
                                                ref int lpnHowMany);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern void TT_GetErrorMessage(int nError,
                                                IntPtr szErrorMsg);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_HotKey_Register(IntPtr lpTTInstance,
                                                int nHotKeyID,
                                                [In, Out] int[] lpnVKCodes,
                                                int nVKCodeCount);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_HotKey_Unregister(IntPtr lpTTInstance,
                                                  int nHotKeyID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_HotKey_IsActive(IntPtr lpTTInstance,
                                                 int nHotKeyID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_HotKey_InstallTestHook(IntPtr lpTTInstance,
            /* HWND */ IntPtr hWnd);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_HotKey_RemoveTestHook(IntPtr lpTTInstance);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_HotKey_GetKeyString(IntPtr lpTTInstance,
                                                    int nVKCode,
                                                    IntPtr szKeyName);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_DBG_SIZEOF(TTStructType nType);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern ClientEvent TT_DBG_EVENT_LAST();

        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetMixerCount();
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_GetMixerName(int nMixerIndex,
                                               IntPtr szMixerName);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_GetWaveInName(int nWaveDeviceID,
                                                IntPtr szMixerName);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_GetWaveOutName(int nWaveDeviceID,
                                                 IntPtr szMixerName);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_SetWaveOutMute(int nWaveDeviceID,
                                                 BearWare.MixerControl nControl,
                                                 bool bMute);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetWaveOutMute(int nWaveDeviceID,
                                                  BearWare.MixerControl nControl);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_SetWaveOutVolume(int nWaveDeviceID,
                                                   BearWare.MixerControl nControl,
                                                   int nVolume);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetWaveOutVolume(int nWaveDeviceID,
                                                    BearWare.MixerControl nControl);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_SetWaveInSelected(int nWaveDeviceID,
                                                    BearWare.MixerControl nControl);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetWaveInSelected(int nWaveDeviceID,
                                                     BearWare.MixerControl nControl);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_SetWaveInVolume(int nWaveDeviceID,
                                                  BearWare.MixerControl nControl,
                                                  int nVolume);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetWaveInVolume(int nWaveDeviceID,
                                                   BearWare.MixerControl nControl);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_SetWaveInBoost(int nWaveDeviceID,
                                                 bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetWaveInBoost(int nWaveDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_SetWaveInMute(int nWaveDeviceID,
                                                bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetWaveInMute(int nWaveDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern int TT_Mixer_GetWaveInControlCount(int nWaveDeviceID);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_GetWaveInControlName(int nWaveDeviceID,
                                                       int nControlIndex,
                                                       IntPtr szDeviceName);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_SetWaveInControlSelected(int nWaveDeviceID,
                                                           int nControlIndex);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Mixer_GetWaveInControlSelected(int nWaveDeviceID,
                                                            int nControlIndex);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Firewall_IsEnabled();
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Firewall_Enable(bool bEnable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Firewall_AppExceptionExists([MarshalAs(UnmanagedType.LPWStr)] string szExecutable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Firewall_AddAppException([MarshalAs(UnmanagedType.LPWStr)] string szName, 
                                                     [MarshalAs(UnmanagedType.LPWStr)] string  szExecutable);
        [DllImport(dllname, CallingConvention = CallingConvention.Cdecl, CharSet = CharSet.Unicode)]
        public static extern bool TT_Firewall_RemoveAppException([MarshalAs(UnmanagedType.LPWStr)] string szExecutable);
    }
}